/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core.objects;

import rjw.dc.console.MessageBox;
import rjw.dc.core.Repository;
import rjw.dc.db.QueryCondition;
import rjw.dc.db.QueryOptions;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Converter;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * Base class for all Data Crow objects. Here the fields, values and required
 * fields are stored. Checks are done to garantee uniqueness and validity
 * of the records.
 *
 * Objects which extend this class need to override the save and load methods
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class DataCrowObject {

    protected int rowNumber = -1;

    protected String[] labels;
    protected int[] maxFieldLength;

    protected int[] valueTypes;
    /** Database field names */
    protected String[] fieldNames;
    /** Content of the fields */
    protected Object[] values;
    /** Required field names */
    protected int[] requiredFields;

    protected String sTableName;
    protected static WorkFlow wf = null;

    /** Indicates of the data should be checked for validity */
    protected boolean bValidation = true;
    /** Indicates if queries should display output */
    private   boolean bSilence = false;

    /** The record ID */
    public static final int _ID = 0;
    
    static {
        if (wf == null) {
             wf = new WorkFlow();
        }        
    }

    /**
     * Sets a value for the given field
     *
     * @param iField    field indicator
     * @param sValue    value as String
     */
    public void setValue(int iField, Object value) {
        if (valueTypes[iField] == Repository.datatypes._INTEGER) {
            try {
                if (value instanceof Integer) {
                    values[iField] = value;
                } else {
                    setValue(iField, new Integer(
                                         Integer.parseInt(value.toString())));
                }
            } catch (Exception exp) {
                // say something ?
                values[iField] = new Integer(0);
            }
        } else {
            String sValue = value.toString();
            if (sValue.length() > maxFieldLength[iField] &&
                maxFieldLength[iField] > 0) {
                values[iField] = sValue.substring(0, maxFieldLength[iField]);
            } else {
                values[iField] = sValue;  
            }
        }
    }
    
    public String[] getLabels() {
        return labels;
    }

    public String getLabel(int columnIndex) {
        return labels[columnIndex];
    }    

    public int[] getMaxFieldLengths() {
        return maxFieldLength;   
    }
    
    public int getMaxFieldLength(int columnIndex) {
        return maxFieldLength[columnIndex];   
    }    
    /**
     * Returns the value of a field
     *
     * @param iField    field indicator
     * @return (String) field value
     */
    public Object getValue(int iField) {
        return values[iField];
    }

    /**
     * Returns all values
     *
     * @return (String[])   all values for the fields
     */
    public Object[] getValues() {
        return values;
    }

    public void setUIRowNumber(int row) {
        rowNumber = row;   
    }

    /**
     * Returns all the database field names
     *
     * @return (String[])   database field names
     */
    public String[] getFieldNames() {
        return fieldNames;
    }
    
    public String getFieldName(int columnIndex) {
        return fieldNames[columnIndex];
    }    

    /**
     * Returns the required field names
     *
     * @return (String[])   required field names
     */
    public int[] getRequiredFields() {
        return requiredFields;
    }


    /**
     * Returns the tablename
     *
     * @return (String) name of the database table
     */
    public String getTableName() {
        return sTableName;
    }
    
    /**
     * Inserts a new record. Requests to the UI can be passed.
     *
     * @param oRequestor    collection of requests, can be null
     */
    public void saveNew(RequestorCollection requestors) {
        if (this.checkIntegrity(false)) {
            wf.insertValues(this, requestors);
        }
    }

    /**
     * Updates a record. Requests to the UI can be passed.
     *
     * @param oRequestor    collection of requests, can be null
     */
    public void saveUpdate(RequestorCollection requestors) {
        if (this.checkIntegrity(true)) {
            wf.updateValues(this, requestors);
        }
    }

    /**
     * Loads records. Requests to the UI can be passed.
     *
     * @param oRequestor    collection of requests, can be null
     * @param options       query options, can be null
     */
    public void load(QueryOptions options, RequestorCollection oRequestor) {
    }    

    /**
     * Disables the output of this query to the log and log screen
     *
     * @param silence   show output y/n
     */
    public void setSilence(boolean silence) {
        this.bSilence = silence;
    }

    /**
     * Indicates if queries should display output
     *
     * return (boolean) show output y/n
     */
    public boolean getSilence() {
        return this.bSilence;
    }

    /**
     * Specifies if values should be validated
     *
     * @param validate  valdate values indicator
     */
    public void setValidation(boolean validate) {
        bValidation = validate;
    }


    /**
     * Creates a query condition for select statements by using the
     * field names and their values
     *
     * @return QueryCondition   query condition definition
     */
    public QueryCondition toQueryCondition(boolean select) {
        QueryCondition condition = new QueryCondition(fieldNames, convertValues(), select);
        return condition;
    }

    /**
     * Checks for illegal characters. Null characters and single quotes are
     * replaced. Converts the characters to UTF8
     *
     * @return (String[])   database ready values
     */
    protected Object[] convertValues() {
        if (values != null) {
            for (int i = 0; i < values.length; i++) {
                if (values[i] instanceof String) {
                    values[i] = Converter.databaseValueConverter(
                                (String) values[i]);
                }
            }
        }
        return values;
    }

    /**
     * Checks if the values are unique and if all the required fields
     * have been filled
     *
     * @param update        indicates if the values are new or existing
     * @return (boolean)    result
     */
    public boolean checkIntegrity(boolean update) {
        boolean bRequiredFieldsFilled = true;
        boolean bUnique = true;

        values = convertValues();

        if (Settings.getValueAsBoolean(Repository.settings.stCheckRequiredFields)) {
            bRequiredFieldsFilled = this.validateRequiredFields();
        }

        if (Settings.getValueAsBoolean(Repository.settings.stCheckUniqueness)) {
            bUnique = this.isUnique(this, update);
        }

        if (bRequiredFieldsFilled && bUnique) {
            return true;
        }

        return false;
    }

    /**
     * Checks to see whether the record already exists or not
     *
     * @param o   object to be checked
     * @param bUpdateQuery  indicates whether the object is to be updated or
     *                      if it is a new object
     */
    public boolean isUnique(DataCrowObject o, boolean bUpdateQuery) {
        if (!bValidation) {
            return true;
        }

        boolean bUnique = wf.checkUniqueness(o, bUpdateQuery);

        if (!bUnique) {
            if (rowNumber != -1) {
                new MessageBox(Resources.getText(DCMessages.msgRecordNotUniqueRow,
                               String.valueOf(rowNumber)),
                               true, 
                               MessageBox._NOTIFICATION);             
            } else {
                new MessageBox(Resources.getText(DCMessages.msgRecordNotUniqueID,
                               getValue(_ID).toString()),
                               true, 
                               MessageBox._NOTIFICATION);             
            }
        }

        return bUnique;
    }

    /**
     * Checks to see whether al required fields have been filled
     *
     * @return (String) string of unfilled fields
     */
    public String getUnfilledRequiredFields() {
        String unFilledFields = "";

        if (values == null) {
            for (int i = 0; i < requiredFields.length; i++) {
                unFilledFields += getFieldName(requiredFields[i]) + " ";
            }
        }

        for (int i = 0; i < requiredFields.length; i++) {
            if (this.getValue(requiredFields[i]) == null || 
                this.getValue(requiredFields[i]).equals("Null") ||
                this.getValue(requiredFields[i]).toString().trim().equals("")) {
                unFilledFields += getFieldName(requiredFields[i]) + " ";
            }
        }

        return unFilledFields;
    }

    /**
     * Check if all required fields have been filled, returns a boolean
     *
     * @return (boolean)    indicator for uniqueness
     */
    private boolean validateRequiredFields() {
        String unFilledFields = this.getUnfilledRequiredFields();

        if (requiredFields == null || !bValidation) {
            return true;
        }

        if (unFilledFields.trim().equals("")) {
            return true;
        } else {
            if (rowNumber != -1) {
                new MessageBox(Resources.getText(DCMessages.msgIncompleteRecRow,
                               new String[] {unFilledFields,
                                             String.valueOf(rowNumber)}),
                               true, 
                               MessageBox._NOTIFICATION);             
            } else {
                new MessageBox(Resources.getText(DCMessages.msgIncompleteRecID, 
                                                 new String[] {unFilledFields,
                                                               getValue(_ID).toString()}),
                               true, 
                               MessageBox._NOTIFICATION);             
            }
                 
            return false;
        }
    }
}
