/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core.objects.movie;

import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.db.QueryOptions;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class Movie extends DataCrowObject {

    public static final int _A_TITLE = 1;
    public static final int _B_DESCRIPTION = 2;
    public static final int _C_WEBPAGE = 3;
    public static final int _D_GENRE = 4;
    public static final int _E_ACTORS = 5;
    public static final int _F_DIRECTOR = 6;
    public static final int _G_COUNTRY = 7;
    public static final int _H_YEAR = 8;
    public static final int _I_PLAYLENGTH = 9;
    public static final int _J_FILENAME = 10;
    public static final int _K_LOCATION = 11;
    public static final int _L_KEEP = 12;

    private static WorkFlow wf;
    
    /**
     * Initializes the Movie object; sets the required fields, the table name,
     * initializes the value array and the database field names
     */
    public Movie() {
       if (wf == null) {
           wf = new WorkFlow();
       }

       valueTypes = Repository.dataBase.movieValueTypes; 
       values = new Object[Repository.dataBase.dbMovieColArrayPK.length];
       sTableName = Repository.dataBase.dBMovieTableName.trim();
       requiredFields = Settings.getValueAsIntegerArray(
                                 Repository.settings.stMovieRequiredFields);
       fieldNames = Repository.dataBase.dbMovieColArrayPK;

       labels = new String[Repository.dataBase.dbMovieColArrayPK.length];
       int i = 0;
       labels[i++] = Resources.getText(DCLabels.lblID); 
       labels[i++] = Resources.getText(DCLabels.lblTitle);
       labels[i++] = Resources.getText(DCLabels.lblDescription);
       labels[i++] = Resources.getText(DCLabels.lblWebPage);
       labels[i++] = Resources.getText(DCLabels.lblGenre);
       labels[i++] = Resources.getText(DCLabels.lblActors);
       labels[i++] = Resources.getText(DCLabels.lblDirector);
       labels[i++] = Resources.getText(DCLabels.lblCountry);
       labels[i++] = Resources.getText(DCLabels.lblYear);       
       labels[i++] = Resources.getText(DCLabels.lblPlayLength); 
       labels[i++] = Resources.getText(DCLabels.lblFileName);
       labels[i++] = Resources.getText(DCLabels.lblLocation); 
       labels[i++] = Resources.getText(DCLabels.lblDispose);    
       
       maxFieldLength = Repository.dataBase.movieFieldLengths;

       for (int j = 0; j < values.length; j++) {
          setValue(j, new String(""));
       }
    }


    /**
     * Applies values to this Movie record. The first in the row is the ID
     *
     * @param row   representation of an database row. First in row is the ID
     */
    public void setValues(Object[] row) {
        int i = 0;

        try {
            if (row.length < Repository.dataBase.dbMovieColArray.length) {
                throw new Exception(Resources.getText(DCMessages.msgInvalidArray));
            } else if (row.length == Repository.dataBase.dbMovieColArrayPK.length) {
                setValue(_ID, row[i++]);
            }

            setValue(_A_TITLE, row[i++]);
            setValue(_B_DESCRIPTION, row[i++]);
            setValue(_C_WEBPAGE, row[i++]);
            setValue(_D_GENRE, row[i++]);
            setValue(_E_ACTORS, row[i++]);
            setValue(_F_DIRECTOR, row[i++]);
            setValue(_G_COUNTRY, row[i++]);
            setValue(_H_YEAR, row[i++]);
            setValue(_I_PLAYLENGTH, row[i++]);
            setValue(_J_FILENAME, row[i++]);
            setValue(_K_LOCATION, row[i++]);
            setValue(_L_KEEP, row[i++]);

        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Saves a new record to the database. Values are first validated for validity and
     * integrity
     */
    public void saveNew(RequestorCollection requestors) {
        if (this.checkIntegrity(false)) {
            wf.insertValues(this, requestors);
        }
    }

    /**
     * Updates an existing record. Values are first validated for validity and
     * integrity
     */
    public void saveUpdate(RequestorCollection requestors) {
        if (this.checkIntegrity(true)) {
            wf.updateValues(this, requestors);
        }
    }

    /**
     * Loads the values from the database. UI requests are past to the workflow
     *
     * @param options       search options
     * @param requestors    displaying request for the UI
     */
    public void load(QueryOptions options, RequestorCollection requestors) {
        wf.searchValues(this, options, requestors);
    }
}
