/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core.objects.mp3;

import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.db.QueryOptions;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * Representation of an MP3 file. Allows saving, updating and querying of MP3 records
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class Mp3 extends DataCrowObject {

    /** MP3 filename */
    public static final int _A_FILENAME = 1;
    /** Song title */
    public static final int _B_TITLE = 2;
    /** Artist name */
    public static final int _C_ARTIST = 3;
    /** Album name */
    public static final int _D_ALBUM = 4;
    /** Year of creation */
    public static final int _E_YEAR = 5;
    /** Genre name */
    public static final int _F_GENRE = 6;
    /** Play length  */
    public static final int _G_LENGTH = 7;
    /** Sound quality */
    public static final int _H_QUALITY = 8;
    /** Location name (for grouping data) */
    public static final int _I_LOCATION = 9;
    /** Keep this record (Y/N) */
    public static final int _J_KEEP = 10;

    private static WorkFlow wf;
    
    /**
     * Initializes the MP3 object; sets the required fields, the table name,
     * initializes the value array and the database field names
     */
    public Mp3() {
       if (wf == null) {
           wf = new WorkFlow();
       }

       valueTypes = Repository.dataBase.mp3ValueTypes;
       values = new Object[Repository.dataBase.dbMP3ColArrayPK.length];
       sTableName = Repository.dataBase.dBMP3TableName.trim();
       requiredFields = Settings.getValueAsIntegerArray(
                                 Repository.settings.stMp3RequiredFields);
       fieldNames = Repository.dataBase.dbMP3ColArrayPK;

       labels = new String[Repository.dataBase.dbMP3ColArrayPK.length];
       int i = 0;
       labels[i++] = Resources.getText(DCLabels.lblID);
       labels[i++] = Resources.getText(DCLabels.lblFileName);
       labels[i++] = Resources.getText(DCLabels.lblTitle);
       labels[i++] = Resources.getText(DCLabels.lblArtist);
       labels[i++] = Resources.getText(DCLabels.lblAlbum);
       labels[i++] = Resources.getText(DCLabels.lblYear);
       labels[i++] = Resources.getText(DCLabels.lblGenre);
       labels[i++] = Resources.getText(DCLabels.lblPlayLength);
       labels[i++] = Resources.getText(DCLabels.lblQuality);       
       labels[i++] = Resources.getText(DCLabels.lblLocation);       
       labels[i++] = Resources.getText(DCLabels.lblDispose);         

       maxFieldLength = Repository.dataBase.mp3FieldLengths;

       for (int j = 0; j < values.length; j++) {
          setValue(j, new String(""));
       }
    }

    /**
     * Applies values to this MP3 record. The first in the row is the ID
     *
     * @param row   representation of an database row. First in row is the ID
     */
    public void setValues(Object[] row) {
        int i = 0;
        try {
            if (row.length < Repository.dataBase.dbMP3ColArray.length) {
                throw new Exception(Resources.getText(DCMessages.msgInvalidArray));
            } else if (row.length == Repository.dataBase.dbMP3ColArrayPK.length) {
                setValue(_ID, row[i++]);
            }

            setValue(_A_FILENAME, row[i++]);
            setValue(_B_TITLE, row[i++]);
            setValue(_C_ARTIST, row[i++]);
            setValue(_D_ALBUM, row[i++]);
            setValue(_E_YEAR, row[i++]);
            setValue(_F_GENRE, row[i++]);
            
            String length = row[i].toString();
            if (length.indexOf("s") > 0) {
                length = length.substring(0, length.indexOf("s")).trim();
                setValue(_G_LENGTH, length);
                i++;
            } else {
                setValue(_G_LENGTH, row[i++]);
            }
            setValue(_H_QUALITY, row[i++]);
            setValue(_I_LOCATION, row[i++]);
            setValue(_J_KEEP, row[i++]);

        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Saves a new record to the database. Values are first validated for validity and
     * integrity
     */
    public void saveNew(RequestorCollection requestors) {
        if (this.checkIntegrity(false)) {
            wf.insertValues(this, requestors);
        }
    }

    /**
     * Updates an existing record. Values are first validated for validity and
     * integrity
     */
    public void saveUpdate(RequestorCollection requestors) {
        if (this.checkIntegrity(true)) {
            wf.updateValues(this, requestors);
        }
    }

    /**
     * Loads the values from the database. UI requests are past to the workflow
     *
     * @param options       search options
     * @param requestors    displaying request for the UI
     */
    public void load(QueryOptions options, RequestorCollection requestors) {
        wf.searchValues(this, options, requestors);
    }
}
