/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core.objects.software;

import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.db.QueryOptions;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * Representation of a Software record. Allows saving, updating and querying
 * of Software records
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class Software extends DataCrowObject {

    /** Name of the Software */
    public static final int _A_NAME = 1;
    /** Description */
    public static final int _B_DESCRIPTION = 2;
    /** Path */
    public static final int _C_FILENAME = 3;
    /** Software Catagory */
    public static final int _D_TYPE = 4;
    /** Multiplayer Y/N */
    public static final int _E_MULTI = 5;
    /** Location of the Software (for grouping) */
    public static final int _F_LOCATION = 6;
    /** Keep this record Y/N */
    public static final int _G_DISPOSE = 7;

    // hack for the database upgrade(s)
    private static boolean bConvertCatagory = true;

    private static WorkFlow wf;

    /**
     * Initializes the Software object; sets the required fields, the table name,
     * initializes the value array and the database field names
     */
    public Software() {
       if (wf == null) {
           wf = new WorkFlow();
       } 

       valueTypes = Repository.dataBase.softwareValueTypes;
       values = new Object[Repository.dataBase.dbSoftColArrayPK.length];
       sTableName = Repository.dataBase.dBSoftTableName.trim();
       requiredFields = Settings.getValueAsIntegerArray(
                                 Repository.settings.stSoftwareRequiredFields);
       fieldNames = Repository.dataBase.dbSoftColArrayPK;
       maxFieldLength =Repository.dataBase.softwareFieldLengths;

       labels = new String[Repository.dataBase.dbSoftColArrayPK.length];
       int i = 0;
       labels[i++] = Resources.getText(DCLabels.lblID);
       labels[i++] = Resources.getText(DCLabels.lblName);
       labels[i++] = Resources.getText(DCLabels.lblDescription);
       labels[i++] = Resources.getText(DCLabels.lblFileName);
       labels[i++] = Resources.getText(DCLabels.lblType);
       labels[i++] = Resources.getText(DCLabels.lblMulti);
       labels[i++] = Resources.getText(DCLabels.lblLocation);
       labels[i++] = Resources.getText(DCLabels.lblDispose);    
       
        for (int j = 0; j < values.length; j++) {
           setValue(j, new String(""));
        }
    }
    
    public static void setConvertCatagory(boolean convert) {
        bConvertCatagory = convert;
    }
    
    /**
     * Applies values to this Software record. The first in the row is the ID
     *
     * @param row   representation of an database row. First in row is the ID
     */
    public void setValues(Object[] row) {
        int i = 0;

        try {
            if (row.length < Repository.dataBase.dbSoftColArray.length) {
                throw new Exception(Resources.getText(DCMessages.msgInvalidArray));
            } else if (row.length == Repository.dataBase.dbSoftColArrayPK.length) {
                setValue(_ID, row[i++]);
            }

            setValue(_A_NAME, row[i++]);
            setValue(_B_DESCRIPTION, row[i++]);
            setValue(_C_FILENAME, row[i++]);
            setValue(_D_TYPE, row[i++]);
            setValue(_E_MULTI, row[i++]);
            setValue(_F_LOCATION, row[i++]);
            setValue(_G_DISPOSE, row[i++]);

        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Applies a value to this Software object
     *
     * @param iField    field number
     * @param sValue    value String
     */
    public void setValue(int iField, Object value) {
        String sValue = value.toString();
        if (iField != _D_TYPE) {
            super.setValue(iField, sValue);
        } else if (sValue != null && !sValue.trim().equals("") &&
                   bConvertCatagory) {
            try {
                Long.parseLong(sValue);
                sValue = CatagoryCollection.getCatagoryName(sValue);
            } catch (Exception exp) {
                sValue = CatagoryCollection.getCatagoryID(sValue);
            }
            super.setValue(iField, sValue);
        } else {
            super.setValue(iField, sValue);
        }
    }

    /**
     * Loads the values from the database. UI requests are past to the workflow
     *
     * @param options       search options
     * @param requestors    displaying request for the UI
     */
    public void load(QueryOptions options, RequestorCollection requestors) {
        wf.searchValues(this, options, requestors);
    }
}
