/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.processes;

import helliker.id3.MP3File;
import java.io.File;
import java.io.RandomAccessFile;
import java.util.Enumeration;
import java.util.Vector;

import rjw.dc.console.MessageBox;
import rjw.dc.console.mp3.InsertMP3Panel;
import rjw.dc.core.DataCrow;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.CdRom;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.mp3.Mp3;
import rjw.dc.messages.Messages;
import rjw.dc.messages.MessagesWriter;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Converter;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * Exports a MP3 CD to a file. Later on it can be imported by  the ImportFile class.
 * The layout of the export file is like this; value <seperator> value.
 * The seperator is retrieved form the <url>ImportInformationFile</url>
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ExportMP3CD extends Thread {

    private CdRom cdRom = new CdRom();
    private String location = "";
    private boolean bConfirmDirs = false;
    private String cdName = "";
    private String file = "";
    private boolean bToTable = false;
    private boolean bToFile = true;
    /** Specifies whether the export process is running */
    public static boolean running = false;

    private static final WorkFlow wf = new WorkFlow();

    /**
     * Initializes the export process. All preferences are set
     *
     * @param loc location from which the data is read
     * @param bConfirmDirs  indicates if per directory should be asked if the data
     *                      should be read from this directory
     * @param file the export filename
     * @param cdName name of the location, used for grouping data
     */
    public ExportMP3CD( String loc,
                        boolean bConfirmDirs,
                        String file,
                        String cdName,
                        boolean bToFile,
                        boolean bToTable) {

        if (!loc.endsWith("\\") && !loc.endsWith("/")) {
            this.location = loc;
        } else {
            this.location = loc.substring(0, loc.length() -1);
        }
        this.bConfirmDirs = bConfirmDirs;
        this.file = file;
        this.cdName = cdName;
        this.bToFile = bToFile;
        this.bToTable = bToTable;
    }

    /**
     * Starts the export process. The data read from the location is written
     * to the export file. Information from the files is separated by..
     * seperators as specified in the IIF. Errors are shown to user in
     * messageboxes
     */
    public void run() {
        try {
            running = true;
            Vector filesAndDirs = cdRom.getInformation( location, 
                                                        bConfirmDirs,
                                                        true,
                                                        false,
                                                         "mp3");
            writeToFile(filesAndDirs);
        } catch (Exception exp) {
            exp.printStackTrace();
            MessagesWriter._useLogFile = true;
            Messages.setMessage(exp.toString(), Messages._ERROR);
            new MessageBox(DataCrow.mainFrame,
                           Resources.getText(DCMessages.msgProcessCrash), 
                           true, MessageBox._ERROR);
            finishAll();
        }
        new MessageBox(DataCrow.mainFrame,
                       Resources.getText(DCMessages.msgExportCompleted), 
                       true, MessageBox._INFORMATION);
        finishAll();
    }

    /**
     * Writes the data to the file. Information from the files is separated by..
     * seperators as specified in the IIF.
     */
    public void writeToFile(Vector data) {
        File sourceFile;

        try {
            try {
                sourceFile = new File(file.trim());
            } catch (Exception exp) {
                sourceFile = new File(Settings.getValueAsString(Repository.settings.stMP3ExportFile));
            }
            new MessageBox(DataCrow.mainFrame,
                           Resources.getText(DCMessages.msgWriteToFile, sourceFile.getName()), 
                           true, MessageBox._INFORMATION);
            RandomAccessFile fileAccess = new RandomAccessFile(sourceFile, "rw");
            Messages.setMessage(Resources.getText(DCMessages.msgWritingInfo), 
                                Messages._STANDARD);

            fileAccess.seek(fileAccess.length());
            fileAccess.writeBytes("->" + Repository.dataBase.dBMP3TableName.trim() + "<-" + '\n');

            boolean first = true;

            for (Enumeration enum = data.elements(); enum.hasMoreElements(); ) {
                String sFilename = (String) enum.nextElement();
                Object[] mp3Info = getMP3Information(sFilename);

                Messages.setMessage(
                    Resources.getText(DCMessages.msgWritingToFile, sFilename),
                    Messages._STANDARD);

                if (first && bToFile) {
                    fileAccess.seek(fileAccess.length());

                    for (int i = 0; i < mp3Info.length; i++) {
                        fileAccess.writeBytes(mp3Info[i] + " > ");
                    }

                    fileAccess.writeBytes("[" + cdName + "]" + '\n');
                    first = false;
                } else  if (bToFile) {
                    try {
                        this.sleep(10);
                    } catch (InterruptedException intExp) {}

                    for (int i = 0; i < mp3Info.length; i++) {
                        fileAccess.writeBytes(mp3Info[i] + " > ");
                    }
                    fileAccess.writeBytes("\n");
                }

                if (bToTable) {
                    Mp3 mp3 = new Mp3();
                    mp3.setValues(mp3Info);
                    mp3.setValue(Mp3._I_LOCATION, cdName);
                    Vector vTable = new Vector(0);
                    vTable.add(mp3);
                    Requestor req = new Requestor(InsertMP3Panel.class,
                                                  "addRow",
                                                  null);
                    RequestorCollection reqCol = new RequestorCollection();
                    reqCol.addRequestor(req);
                    wf.displayResults(vTable, reqCol);
                }
            }

            Messages.setMessage(
                    Resources.getText(DCMessages.msgCreatingIIF), 
                    Messages._STANDARD);

            String[][] IIFData = { {Repository.dataBase.dBMP3ColFileName, ">" },
                                   {Repository.dataBase.dBMP3ColTitle, ">" },
                                   {Repository.dataBase.dBMP3ColArtist, ">" },
                                   {Repository.dataBase.dBMP3ColAlbum, ">" },
                                   {Repository.dataBase.dBMP3ColYear, ">" },
                                   {Repository.dataBase.dBMP3ColGenre, ">" },
                                   {Repository.dataBase.dBMP3ColLength, ">" },
                                   {Repository.dataBase.dBMP3ColQuality, ">" },
                                   {Repository.dataBase.dBMP3ColLocation, ">" },
                                   {Repository.dataBase.dBMP3ColDispose, ">" },
            };

            new ImportInformationFile(IIFData, file);
            Messages.setMessage(
                Resources.getText(DCMessages.msgCreatedIIF), 
                Messages._STANDARD);

            fileAccess.close();

        } catch (Exception exp) {
            MessagesWriter._useLogFile = true;
            new MessageBox(DataCrow.mainFrame,
                           Resources.getText(DCMessages.msgProcessCrash), 
                           true, MessageBox._ERROR);
            Messages.setMessage(exp.toString(), Messages._ERROR);
            finishAll();
        }
    }

    /**
     * Stops the process
     */
    private void finishAll() {
        try {
            running = false;
            MessagesWriter._useLogFile = true;
            Messages.setMessage(
                Resources.getText(DCMessages.msgExportStops), 
                Messages._STANDARD);
        } catch (Exception exp) {
        }
    }

    /**
     * Retrieves information from the ID3 tags and ID3v2 tags of the MP3 file.
     * An external package is used for getting this information
     *
     * @param sFile         the Mp3 filename with absolute path
     * @return (String[])   retrieved information
     */
    private Object[] getMP3Information(String sFile) {
        Object[] mp3Info = new Object[10];
        try {
            MP3File mp3File     = new MP3File(sFile);
            String sFileName    = (String) toValidString(sFile);

            int i = 0;
            mp3Info[i++] = sFileName.substring(2, sFileName.length());
            mp3Info[i++] = toValidString(mp3File.getTitle());
            mp3Info[i++] = toValidString(mp3File.getArtist());
            mp3Info[i++] = toValidString(mp3File.getAlbum());
            mp3Info[i++] = toValidString(mp3File.getYear());
            mp3Info[i++] = toValidString(mp3File.getGenre());
            mp3Info[i++] = new Integer(new Long(mp3File.getPlayingTime()).intValue());
            mp3Info[i++] = new Integer(mp3File.getBitRate());
            mp3Info[i++] = toValidString(cdName);
            mp3Info[i++] = toValidString(Resources.getText(DCLabels.lblYes));

        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
        return mp3Info;
    }

    /**
     * Null values are converted to the database standard, single quotes are converted
     * and null characters are converted to spaces
     *
     * @param sValue    value to be checked
     * @return (String) value which complies with the database standard
     */         
    protected String toValidString(String value) {
        return Converter.databaseValueConverter(value);
    }
}
